import boto3

from cloud_governance.common.aws.iam.iam_operations import IAMOperations
from cloud_governance.common.elasticsearch.elasticsearch_operations import ElasticSearchOperations
from cloud_governance.common.logger.init_logger import logger


class ValidateIAMUserTags:

    def __init__(self, es_host: str, es_port: str, es_index: str):
        self.__es_host = es_host
        self.__es_port = es_port
        self.__es_index = es_index
        if self.__es_host and self.__es_port:
            self.__elastic_search_operations = ElasticSearchOperations(es_host=self.__es_host, es_port=self.__es_port)
        self.iam_client = boto3.client('iam')
        self.iam_operations = IAMOperations()

    def check_trail_spaces(self, tags: list, ):
        """
        Check trail spaces
        @param tags:
        @return:
        """
        new_tags = []
        if tags:
            for tag in tags:
                if tag.get('Key').startswith(' ') or tag.get('Key').endswith(' '):
                    new_tags.append(tag.get('Key').strip())
        return new_tags

    def upload_trailing_user_tags(self):
        """
        This method gets the trailing spaces tags of each user
        @return:
        """
        users = self.iam_operations.get_users()
        output_data = []
        for user in users:
            if '-' not in user['UserName']:
                tags = self.iam_operations.get_user_tags(username=user['UserName'])
                trailing_tags = self.check_trail_spaces(tags)
                if trailing_tags:
                    output_data.append({'User': user['UserName'], 'TrailingSpaces': trailing_tags})
                    if self.__es_host and self.__es_port:
                        self.__elastic_search_operations.upload_to_elasticsearch(index=self.__es_index, data={'User': user['UserName'], 'TrailingSpaces': trailing_tags})
        logger.info(f'Trailing Spaces User: {output_data}')

    def __check_tags(self, tags: list, mandatory_tags: list):
        """
        This method returns the count of which user don't have the mandatory_tags
        @param tags:
        @param mandatory_tags:
        @return:
        """
        present_tags = []
        if tags:
            for tag in tags:
                if tag.get('Key') in mandatory_tags:
                    present_tags.append(tag.get('Key'))
        return list(set(mandatory_tags) - set(present_tags))

    def upload_user_without_mandatory_tags(self, mandatory_tags: list):
        """
        This method gets list users which don't have all mandatory tags
        @param mandatory_tags:
        @return:
        """
        users = self.iam_operations.get_users()
        output_data = []
        for user in users:
            username = user.get('UserName')
            if '-' not in username:
                tags = self.iam_operations.get_user_tags(username=user['UserName'])
                missing_tags = self.__check_tags(tags=tags, mandatory_tags=mandatory_tags)
                if missing_tags:
                    output_data.append({'User': username, 'MissingTags': missing_tags})
                    if self.__es_host and self.__es_port:
                        self.__elastic_search_operations.upload_to_elasticsearch(index=self.__es_index, data={'User': username, 'MissingTags': missing_tags})
        logger.info(f'Missing tags Users:: {output_data}')



import json
from multiprocessing import Process

from cloud_governance.common.aws.cost_explorer.cost_explorer_operations import CostExplorerOperations
from cloud_governance.common.logger.init_logger import logger
from cloud_governance.common.elasticsearch.elasticsearch_operations import ElasticSearchOperations


class GenerateCostExplorerReport:

    def __init__(self, cost_tags: list, granularity: str = 'DAILY', es_host: str = '', es_port: str = '', es_index: str = '', cost_metric: str = 'UnblendedCost', file_name: str = '', start_date: str = '', end_date: str = ''):
        self.start_date = start_date
        self.end_date = end_date
        self.granularity = granularity
        self.cost_metric = cost_metric
        self.cost_tags = cost_tags
        self.__cost_explorer = CostExplorerOperations()
        self.file_name = file_name
        self.__es_host = es_host
        self.__es_port = es_port
        self.__es_index = es_index
        self.__elastic_search_operations = ElasticSearchOperations(es_host=self.__es_host, es_port=self.__es_port)

    def filter_data_by_tag(self, groups: list, tag: str):
        """
        This method extract data by tag
        @param tag:
        @param groups: Data from the cloud explorer
        @return: converted into dict format
        """
        data = []
        for group in groups:
            name = ''
            amount = ''
            if group.get('Keys'):
                name = group.get('Keys')[0].split('$')[-1]
                name = name if name else 'NoTagKey'
            if group.get('Metrics'):
                amount = group.get('Metrics').get(self.cost_metric).get('Amount')
            if name and amount:
                data.append({tag: name, 'Cost': round(float(amount), 3)})
        return data

    def __get_daily_cost_by_tags(self):
        """
        This method extracts the costs by tags and upload to elastic search
        @return:
        """
        data_house = {}
        for tag in self.cost_tags:
            if self.start_date and self.end_date:
                response = self.__cost_explorer.get_cost_by_tags(tag=tag, start_date=self.start_date, end_date=self.end_date, granularity=self.granularity, cost_metric=self.cost_metric)
            else:
                response = self.__cost_explorer.get_cost_by_tags(tag=tag, granularity=self.granularity, cost_metric=self.cost_metric)
            results_by_time = response.get('ResultsByTime')
            if results_by_time:
                data_house[tag] = []
                for result in results_by_time:
                    data_house[tag].extend(self.filter_data_by_tag(result.get('Groups'), tag))
        return data_house

    def __upload_data(self, data: list, index: str):
        """
        This method upload to elastic search
        @param data:
        @param index:
        @return:
        """
        if self.file_name:
            with open(f'/tmp/{self.file_name}', 'a') as file:
                for value in data:
                    file.write(f'{value}\n')
        else:
            for value in data:
                self.__elastic_search_operations.upload_to_elasticsearch(index=index, data=value)
        logger.info(f'Data uploaded to {index}')

    def upload_tags_cost_to_elastic_search(self):
        """
        This method upload daily tag cost into ElasticSearch
        @return:
        """
        logger.info(f'Get {self.granularity} Cost usage by metric: {self.cost_metric}')
        cost_data = self.__get_daily_cost_by_tags()
        jobs = []
        for key, values in cost_data.items():
            index = f'{self.__es_index}-{key.lower()}'
            p = Process(target=self.__upload_data, args=(values, index, ))
            p.start()
            jobs.append(p)
        for job in jobs:
            job.join()



from datetime import datetime

import boto3

from cloud_governance.common.aws.cloudtrail.cloudtrail_operations import CloudTrailOperations
from cloud_governance.common.aws.ec2.ec2_operations import EC2Operations
from cloud_governance.common.aws.iam.iam_operations import IAMOperations
from cloud_governance.common.aws.utils.utils import Utils
from cloud_governance.common.logger.init_logger import logger


class RemoveNonClusterTags:

    def __init__(self, region: str = 'us-east-2', dry_run: str = 'yes', input_tags: dict = ''):
        self.region = region
        self.dry_run = dry_run
        self.input_tags = input_tags
        self.cloudtrail = boto3.client('cloudtrail', region_name=region)
        self.cluster_prefix = 'kubernetes.io/cluster/'
        self.ec2_client = boto3.client('ec2', region_name=region)
        self.cloudtrail = CloudTrailOperations(region_name=self.region)
        self.iam_client = IAMOperations()
        self.ec2_operations = EC2Operations(region=region)
        self.utils = Utils(region=region)

    def __get_instances_data(self, instance_id: str = ''):
        """
        This method go over all instances
        :return:
        """
        ec2s_data = self.ec2_operations.get_instances()
        if instance_id:
            for items in ec2s_data:
                if items.get('Instances'):
                    for item in items['Instances']:
                        if item.get('InstanceId') == instance_id:
                            return items['Instances']
        else:
            instances_list = []
            for items in ec2s_data:
                if items.get('Instances'):
                    instances_list.append(items['Instances'])
            return instances_list

    def __append_input_tags(self):
        """
        This method build tags list according to input tags dictionary
        @return:
        """
        tags_list = []
        for key, value in self.input_tags.items():
            tags_list.append({'Key': key, 'Value': value})
        return tags_list

    def __get_tags_of_resources(self, tags: list, search_tags: list):
        """
        This method extracts tags from the resource tags
        @param tags:
        @param search_tags:
        @return:
        """
        add_tags = []
        if tags:
            for search_tag in search_tags:
                found = False
                for tag in tags:
                    if tag.get('Key') == search_tag.get('Key'):
                        found = True
                if not found:
                    add_tags.append(search_tag)
        else:
            add_tags.extend(search_tags)
        return add_tags

    def __get_instance_tags(self, launch_time: datetime, instance_id: str, tags: list):
        """
        This method returns the tags to update  the instance tags
        @param launch_time:
        @param instance_id:
        @param tags:
        @return:
        """
        username = self.cloudtrail.get_username_by_instance_id_and_time(launch_time, instance_id, 'AWS::EC2::Instance')
        search_tags = []
        user_tags = self.iam_client.get_user_tags(username=username)
        if not username:
            search_tags.append({'Key': 'User', 'Value': 'NA'})
            search_tags.append({'Key': 'Manager', 'Value': 'NA'})
            search_tags.append({'Key': 'Project', 'Value': 'NA'})
            search_tags.append({'Key': 'Email', 'Value': 'NA'})
            search_tags.append({'Key': 'Environment', 'Value': 'NA'})
            search_tags.append(({'Key': 'Owner', 'Value': 'NA'}))
        else:
            search_tags.append({'Key': 'Email', 'Value': f'{username}@redhat.com'})
        search_tags.extend([{'Key': 'LaunchTime', 'Value': launch_time.strftime('%Y/%m/%d %H:%M:%S')}])
        search_tags.extend(self.__append_input_tags())
        search_tags.extend(user_tags)
        return search_tags

    def non_cluster_update_ec2(self, instances_list: list = None):
        """
        This method removes the tags of ec2 instances
        @return:
        """
        if not instances_list:
            instances_list = self.__get_instances_data()
            _, instances_list = self.ec2_operations.scan_cluster_or_non_cluster_instance(instances_list)
        instances_ids = []
        for instance in instances_list:
            for item in instance:
                instance_id = item.get('InstanceId')
                launch_time = item.get('LaunchTime')
                add_tags = self.__get_instance_tags(launch_time=launch_time, instance_id=instance_id,
                                                    tags=item.get('Tags'))
                if add_tags:
                    if self.dry_run == 'no':
                        self.ec2_client.delete_tags(Resources=[instance_id], Tags=add_tags)
                        logger.info(f'delete tags of instance: {instance_id} total: {len(add_tags)} tags: {add_tags}')
                    instances_ids.append(instance_id)
        logger.info(f'non_cluster_ec2 count: {len(sorted(instances_ids))} {sorted(instances_ids)}')
        return sorted(instances_ids)

    def update_volumes(self, volumes_data: list = None):
        """
        This method removes the tags of non-cluster volumes
        @param volumes_data:
        @return:
        """
        if not volumes_data:
            volumes_data = self.ec2_operations.get_volumes()
            _, volumes_data = self.ec2_operations.scan_cluster_non_cluster_resources(volumes_data)
        volume_ids = []
        for volume in volumes_data:
            volume_id = volume.get('VolumeId')
            tag_name = ''
            username = self.cloudtrail.get_username_by_instance_id_and_time(volume.get('CreateTime'), volume_id,
                                                                            'AWS::EC2::Volume')
            search_tags = []
            if not username:
                if volume.get('Attachments'):
                    for attachment in volume.get('Attachments'):
                        for item in self.__get_instances_data(attachment.get('InstanceId')):
                            if item.get('tags'):
                                search_tags.extend(
                                    [tag for tag in item.get('Tags') if not tag.get('Key') == 'Name'])
                                for tag in item.get('Tags'):
                                    if tag.get('Key') == 'User':
                                        username = tag.get('Key')
                                    elif tag.get('Key') == 'LaunchTime':
                                        search_tags.append({'Key': 'LaunchTime', 'Value': tag.get('Key')})
                            else:
                                search_tags.extend(self.__append_input_tags())
                                username = self.cloudtrail.get_username_by_instance_id_and_time(
                                    item.get('LaunchTime'), item.get('InstanceId'), 'AWS::EC2::Instance')
                                search_tags.append({'Key': 'LaunchTime',
                                                    'Value': item.get('LaunchTime').strftime('%Y/%m/%d %H:%M:%S')})
                            break
                else:
                    search_tags.extend(self.__append_input_tags())
            else:
                search_tags.extend(self.__append_input_tags())
            if username and not tag_name:
                user_tags = self.iam_client.get_user_tags(username=username)
                search_tags.extend(user_tags)
                search_tags.append({'Key': 'Email', 'Value': f'{username}@redhat.com'})
                search_tags.append(
                    {'Key': 'LaunchTime', 'Value': volume.get('CreateTime').strftime('%Y/%m/%d %H:%M:%S')})
            else:
                search_tags.append({'Key': 'User', 'Value': 'NA'})
                search_tags.append({'Key': 'Manager', 'Value': 'NA'})
                search_tags.append({'Key': 'Project', 'Value': 'NA'})
                search_tags.append({'Key': 'Email', 'Value': 'NA'})
                search_tags.append({'Key': 'Environment', 'Value': 'NA'})
                search_tags.append(({'Key': 'Owner', 'Value': 'NA'}))
                search_tags.extend(self.__append_input_tags())
                search_tags.append(
                    {'Key': 'LaunchTime', 'Value': volume.get('CreateTime').strftime('%Y/%m/%d %H:%M:%S')})
            if search_tags:
                if self.dry_run == 'no':
                    self.ec2_client.delete_tags(Resources=[volume_id], Tags=search_tags)
                    logger.info(f'Remove tags of volume_id: {volume_id} total: {len(search_tags)}  tags: {search_tags}')
                volume_ids.append(volume_id)
        logger.info(f'non_cluster_volumes count: {len(sorted(volume_ids))} {sorted(volume_ids)}')
        return sorted(volume_ids)

    def update_snapshots(self, snapshots: list = None):
        """
        This method removes the tags of  non-cluster snapshots
        @param snapshots:
        @return:
        """
        if not snapshots:
            snapshots = self.ec2_operations.get_snapshots()
            _, snapshots = self.ec2_operations.scan_cluster_non_cluster_resources(snapshots)
        snapshot_ids = []
        for snapshot in snapshots:
            snapshot_id = snapshot.get('SnapshotId')
            username = self.cloudtrail.get_username_by_instance_id_and_time(snapshot.get('StartTime'), snapshot_id,
                                                                            'AWS::EC2::Snapshot')
            search_tags = []
            if not username:
                if snapshot.get('Description') and 'Created' in snapshot.get('Description'):
                    image_id = snapshot.get('Description').split(" ")[-1]
                    images = self.ec2_client.describe_images(Owners=['self'])['Images']
                    for image in images:
                        if image.get('ImageId') == image_id:
                            if image.get('tags'):
                                search_tags.extend(
                                    [tag for tag in image.get('Tags') if not tag.get('Key') == "Name"])
                            else:
                                search_tags.extend(self.__append_input_tags())
                            start_time = datetime.fromisoformat(image.get('CreationDate')[:-1] + '+00:00')
                            username = self.cloudtrail.get_username_by_instance_id_and_time(start_time=start_time,
                                                                                            resource_id=image_id,
                                                                                            resource_type='AWS::EC2::Ami')
                            break
                    if not username:
                        instance_id = snapshot.get('Description').split(" ")[2].split("(")[1][:-1]
                        instances = self.__get_instances_data(instance_id)
                        if instances:
                            for item in instances:
                                if item.get('InstanceId') == instance_id:
                                    if item.get('Tags'):
                                        search_tags.extend(
                                            [tag for tag in item.get('Tags') if not tag.get('Key') == 'Name'])
                                    else:
                                        search_tags.extend(self.__append_input_tags())
                                    username = self.cloudtrail.get_username_by_instance_id_and_time(
                                        item.get('LaunchTime'),
                                        item.get('InstanceId'),
                                        'AWS::EC2::Instance')
                                    search_tags.append({'Key': 'LaunchTime', 'Value': item.get('LaunchTime').strftime('%Y/%m/%d %H:%M:%S')})
                                    break
            else:
                search_tags.extend(self.__append_input_tags())
            if username:
                user_tags = self.iam_client.get_user_tags(username=username)
                search_tags.extend(user_tags)
                search_tags.append({'Key': 'Email', 'Value': f'{username}@redhat.com'})
                search_tags.append(
                    {'Key': 'LaunchTime', 'Value': snapshot.get('StartTime').strftime('%Y/%m/%d %H:%M:%S')})
            else:
                search_tags.append({'Key': 'User', 'Value': 'NA'})
                search_tags.append({'Key': 'Manager', 'Value': 'NA'})
                search_tags.append({'Key': 'Project', 'Value': 'NA'})
                search_tags.append({'Key': 'Email', 'Value': 'NA'})
                search_tags.append({'Key': 'Environment', 'Value': 'NA'})
                search_tags.append(({'Key': 'Owner', 'Value': 'NA'}))
                search_tags.extend(self.__append_input_tags())
                search_tags.append(
                    {'Key': 'LaunchTime', 'Value': snapshot.get('StartTime').strftime('%Y/%m/%d %H:%M:%S')})
            if search_tags:
                if self.dry_run == 'no':
                    self.ec2_client.delete_tags(Resources=[snapshot_id], Tags=search_tags)
                    logger.info(
                        f'Remove tags of snapshots: {snapshot_id} total: {len(search_tags)} tags: {search_tags}')
                snapshot_ids.append(snapshot_id)
        logger.info(f'non_cluster_snapshot count: {len(sorted(snapshot_ids))} {sorted(snapshot_ids)}')
        return sorted(snapshot_ids)

    def update_ami(self, images: list = None):
        """
        This method removes the tags of non-cluster Amazon Machine Images
        @param images:
        @return:
        """
        if not images:
            images = self.ec2_operations.get_images()
            _, images = self.ec2_operations.scan_cluster_non_cluster_resources(images)
        image_ids = []
        for image in images:
            image_id = image.get('ImageId')
            start_time = datetime.fromisoformat(image.get('CreationDate')[:-1] + '+00:00')
            username = self.cloudtrail.get_username_by_instance_id_and_time(start_time=start_time,
                                                                            resource_id=image_id,
                                                                            resource_type='AWS::EC2::Ami')
            search_tags = []
            search_tags.extend(self.__append_input_tags())
            if username:
                user_tags = self.iam_client.get_user_tags(username=username)
                search_tags.extend(user_tags)
                search_tags.append({'Key': 'Email', 'Value': f'{username}@redhat.com'})
            else:
                search_tags.append({'Key': 'User', 'Value': 'NA'})
                search_tags.append({'Key': 'Manager', 'Value': 'NA'})
                search_tags.append({'Key': 'Project', 'Value': 'NA'})
                search_tags.append({'Key': 'Email', 'Value': 'NA'})
                search_tags.append({'Key': 'Environment', 'Value': 'NA'})
                search_tags.append(({'Key': 'Owner', 'Value': 'NA'}))
            search_tags.extend([{'Key': 'LaunchTime', 'Value': start_time.strftime('%Y/%m/%d %H:%M:%S')}])
            if search_tags:
                if self.dry_run == 'no':
                    self.ec2_client.delete_tags(Resources=[image_id], Tags=search_tags)
                    logger.info(f'Remove tags of image: {image_id} total: {len(search_tags)} tags: {search_tags}')
                image_ids.append(image_id)
        logger.info(f'non_cluster_amis count: {len(sorted(image_ids))} {sorted(image_ids)}')
        return sorted(image_ids)
